@@ -24,7 +24,7 @@ from SigProfilerMatrixGenerator.scripts import SigProfilerMatrixGeneratorFunc as
 from SigProfilerMatrixGenerator.scripts import CNVMatrixGenerator as scna
 import sigProfilerPlotting
 import os,sys
-from PyPDF2 import PdfFileMerger
+from PyPDF2 import PdfMerger
 import fitz
 import time
 
@@ -48,7 +48,7 @@ def generate_sample_reconstruction(cosmic_sigs, samples_input, activities, outpu
     project = "test_run"
     mtype="96"
 
-    final_pdf = PdfFileMerger()
+    final_pdf = PdfMerger()
     samples = samples_input.copy(deep=True)
     samples.reset_index(inplace=True)
     for sample_name in samples.columns[1:]:
@@ -92,7 +92,10 @@ def record_parameters(sysdata,execution_parameters,start_time):
     sysdata.write("INPUT DATA\n")
     sysdata.write("\tinput_type: {}\n".format(execution_parameters["input_type"]))
     sysdata.write("\toutput: {}\n".format(execution_parameters["output"]))
-    sysdata.write("\tsamples: {}\n".format(execution_parameters["samples"]))
+    if isinstance(execution_parameters["samples"], str):
+        sysdata.write("\tsamples: {}\n".format(execution_parameters["samples"]))
+    else:
+        sysdata.write("\tsamples: {}\n".format(type(execution_parameters["samples"])))
     sysdata.write("\treference_genome: {}\n".format(execution_parameters["reference_genome"]))
     sysdata.write("\tcontext_types: {}\n".format(execution_parameters["context_type"]))
     sysdata.write("\texome: {}\n".format(execution_parameters["exome"]))
@@ -114,7 +117,8 @@ def record_parameters(sysdata,execution_parameters,start_time):
 def spa_analyze(samples, output, input_type='matrix', context_type="96", signatures=None, signature_database=None,decompose_fit_option= True,denovo_refit_option=True,cosmic_fit_option=True, nnls_add_penalty=0.05, 
               nnls_remove_penalty=0.01, initial_remove_penalty=0.05, de_novo_fit_penalty=0.02, 
               genome_build="GRCh37", cosmic_version=3.3, make_plots=True, collapse_to_SBS96=True,connected_sigs=True, verbose=False,devopts=None,new_signature_thresh_hold=0.8,
-              exclude_signature_subgroups=None, exome=False, export_probabilities=True, export_probabilities_per_mutation=False, sample_reconstruction_plots=False):
+              exclude_signature_subgroups=None, exome=False, export_probabilities=True, export_probabilities_per_mutation=False, sample_reconstruction_plots=False,
+              make_metadata=True):
 
     
     """
@@ -199,35 +203,35 @@ def spa_analyze(samples, output, input_type='matrix', context_type="96", signatu
         m_for_subgroups = 'ID'
 
 
-    default_subgroups_dict= {'remove_MMR_deficiency_signatures' :False,
-                      'remove_POL_deficiency_signatures' :False,
-                      'remove_HR_deficiency_signatures' :False,
-                      'remove_BER_deficiency_signatures' :False,
-                      'remove_Chemotherapy_signatures' :False,
-                      'remove_Immunosuppressants_signatures' :False,
-                      'remove_Treatment_signatures' :False,
-                      'remove_APOBEC_signatures' :False,
-                      'remove_Tobacco_signatures' :False,
-                      'remove_UV_signatures' :False,
-                      'remove_AA_signatures' :False,
-                      'remove_Colibactin_signatures' :False,
-                      'remove_Artifact_signatures' :False,
-                      'remove_Lymphoid_signatures' :False}
+    default_subgroups_dict= {'MMR_deficiency_signatures' :False,
+                      'POL_deficiency_signatures' :False,
+                      'HR_deficiency_signatures' :False,
+                      'BER_deficiency_signatures' :False,
+                      'Chemotherapy_signatures' :False,
+                      'Immunosuppressants_signatures' :False,
+                      'Treatment_signatures' :False,
+                      'APOBEC_signatures' :False,
+                      'Tobacco_signatures' :False,
+                      'UV_signatures' :False,
+                      'AA_signatures' :False,
+                      'Colibactin_signatures' :False,
+                      'Artifact_signatures' :False,
+                      'Lymphoid_signatures' :False}
                       
-    default_subgroups_siglists= {'remove_MMR_deficiency_signatures' :{'SBS':['6', '14', '15', '20', '21', '26', '44'], 'DBS':['7', '10'], 'ID':['7']},
-                      'remove_POL_deficiency_signatures' :{'SBS':['10a', '10b', '10c', '10d', '28'], 'DBS':['3'], 'ID':[]},
-                      'remove_HR_deficiency_signatures' :{'SBS':['3'], 'DBS':[], 'ID':['6']},
-                      'remove_BER_deficiency_signatures' :{'SBS':['30','36'], 'DBS':[], 'ID':[]},
-                      'remove_Chemotherapy_signatures' :{'SBS':['11','25','31','35','86','87','90'], 'DBS':['5'], 'ID':[]},
-                      'remove_Immunosuppressants_signatures' :{'SBS':['32'], 'DBS':[], 'ID':[]},
-                      'remove_Treatment_signatures' :{'SBS':['11','25','31','32','35','86','87','90'], 'DBS':['5'], 'ID':[]},
-                      'remove_APOBEC_signatures' :{'SBS':['2','13'], 'DBS':[], 'ID':[]},
-                      'remove_Tobacco_signatures' :{'SBS':['4','29','92'], 'DBS':['2'], 'ID':['3']},
-                      'remove_UV_signatures' :{'SBS':['7a','7b','7c','7d','38'], 'DBS':['1'], 'ID':['13']},
-                      'remove_AA_signatures' :{'SBS':['22'], 'DBS':[], 'ID':[]},
-                      'remove_Colibactin_signatures' :{'SBS':['88'], 'DBS':[], 'ID':['18']},
-                      'remove_Artifact_signatures' :{'SBS':['27','43','45','46','47','48','49','50','51','52','53','54','55','56','57','58','59','60','95'], 'DBS':[], 'ID':[]},
-                      'remove_Lymphoid_signatures' :{'SBS':['9','84','85'], 'DBS':[], 'ID':[]}}
+    default_subgroups_siglists= {'MMR_deficiency_signatures' :{'SBS':['6', '14', '15', '20', '21', '26', '44'], 'DBS':['7', '10'], 'ID':['7']},
+                      'POL_deficiency_signatures' :{'SBS':['10a', '10b', '10c', '10d', '28'], 'DBS':['3'], 'ID':[]},
+                      'HR_deficiency_signatures' :{'SBS':['3'], 'DBS':[], 'ID':['6']},
+                      'BER_deficiency_signatures' :{'SBS':['30','36'], 'DBS':[], 'ID':[]},
+                      'Chemotherapy_signatures' :{'SBS':['11','25','31','35','86','87','90'], 'DBS':['5'], 'ID':[]},
+                      'Immunosuppressants_signatures' :{'SBS':['32'], 'DBS':[], 'ID':[]},
+                      'Treatment_signatures' :{'SBS':['11','25','31','32','35','86','87','90'], 'DBS':['5'], 'ID':[]},
+                      'APOBEC_signatures' :{'SBS':['2','13'], 'DBS':[], 'ID':[]},
+                      'Tobacco_signatures' :{'SBS':['4','29','92'], 'DBS':['2'], 'ID':['3']},
+                      'UV_signatures' :{'SBS':['7a','7b','7c','7d','38'], 'DBS':['1'], 'ID':['13']},
+                      'AA_signatures' :{'SBS':['22'], 'DBS':[], 'ID':[]},
+                      'Colibactin_signatures' :{'SBS':['88'], 'DBS':[], 'ID':['18']},
+                      'Artifact_signatures' :{'SBS':['27','43','45','46','47','48','49','50','51','52','53','54','55','56','57','58','59','60','95'], 'DBS':[], 'ID':[]},
+                      'Lymphoid_signatures' :{'SBS':['9','84','85'], 'DBS':[], 'ID':[]}}
     
     
     signature_subgroups_dict = default_subgroups_dict.copy()
@@ -260,18 +264,6 @@ def spa_analyze(samples, output, input_type='matrix', context_type="96", signatu
 
     start_time = datetime.datetime.now() # start time
 
-    # create JOB_METADATA
-    sysdata = open(output+"/JOB_METADATA.txt", "w")
-    sysdata.write("THIS FILE CONTAINS THE METADATA ABOUT SYSTEM AND RUNTIME\n\n\n")
-    sysdata.write("-------System Info-------\n")
-    sysdata.write("Operating System Name: "+ platform.uname()[0]+"\n"+"Nodename: "+platform.uname()[1]+"\n"+"Release: "+platform.uname()[2]+"\n"+"Version: "+platform.uname()[3]+"\n")
-    sysdata.write("\n-------Python and Package Versions------- \n")
-    sysdata.write("Python Version: "+str(platform.sys.version_info.major)+"."+str(platform.sys.version_info.minor)+"."+str(platform.sys.version_info.micro)+"\n")
-    sysdata.write("SigProfilerPlotting Version: "+sigProfilerPlotting.__version__+"\n")
-    sysdata.write("SigProfilerMatrixGenerator Version: "+SigProfilerMatrixGenerator.__version__+"\n")
-    sysdata.write("SigProfilerAssignment Version: "+SigProfilerAssignment.__version__+"\n")
-    sysdata.write("Pandas version: "+pd.__version__+"\n")
-    sysdata.write("Numpy version: "+np.__version__+"\n")
     execution_parameters = {"input_type":input_type,
                         "context_type":context_type,
                         "signatures":signatures, 
@@ -293,8 +285,22 @@ def spa_analyze(samples, output, input_type='matrix', context_type="96", signatu
                         "export_probabilities":export_probabilities,
                         "make_plots":make_plots
                         }
-    record_parameters(sysdata,execution_parameters,start_time)
-    sysdata.close()
+
+    if make_metadata:
+        # create JOB_METADATA_SPA
+        sysdata = open(os.path.join(output,"JOB_METADATA_SPA.txt"), "w")
+        sysdata.write("THIS FILE CONTAINS THE METADATA ABOUT SYSTEM AND RUNTIME\n\n\n")
+        sysdata.write("-------System Info-------\n")
+        sysdata.write("Operating System Name: "+ platform.uname()[0]+"\n"+"Nodename: "+platform.uname()[1]+"\n"+"Release: "+platform.uname()[2]+"\n"+"Version: "+platform.uname()[3]+"\n")
+        sysdata.write("\n-------Python and Package Versions------- \n")
+        sysdata.write("Python Version: "+str(platform.sys.version_info.major)+"."+str(platform.sys.version_info.minor)+"."+str(platform.sys.version_info.micro)+"\n")
+        sysdata.write("SigProfilerPlotting Version: "+sigProfilerPlotting.__version__+"\n")
+        sysdata.write("SigProfilerMatrixGenerator Version: "+SigProfilerMatrixGenerator.__version__+"\n")
+        sysdata.write("SigProfilerAssignment Version: "+SigProfilerAssignment.__version__+"\n")
+        sysdata.write("Pandas version: "+pd.__version__+"\n")
+        sysdata.write("Numpy version: "+np.__version__+"\n")
+        record_parameters(sysdata,execution_parameters,start_time)
+        sysdata.close()
     # Add sequence parameter to control the tmbplot y-axis scale
     if exome == True:
         sequence = 'exome'
@@ -366,8 +372,9 @@ def spa_analyze(samples, output, input_type='matrix', context_type="96", signatu
         denovo_exposureAvg = np.array(exposureAvg.T)
         print("\n De Novo Fitting .....")
         current_time_start = datetime.datetime.now()
-        with open(output+"JOB_METADATA.txt","a") as sysdata:
-            sysdata.write("\n De Novo Fitting .....")
+        if make_metadata:
+            with open(os.path.join(output,"JOB_METADATA_SPA.txt"),"a") as sysdata:
+                sysdata.write("\n De Novo Fitting .....")
 
 
         #exposureAvg = sub.make_final_solution(processAvg, genomes, listOfSignatures, layer_directory1, mutation_type, index,\
@@ -410,10 +417,10 @@ def spa_analyze(samples, output, input_type='matrix', context_type="96", signatu
                                     initial_remove_penalty=init_rem_denovo,connected_sigs=connected_sigs,refit_denovo_signatures=True,export_probabilities=export_probabilities,
                                     export_probabilities_per_mutation=export_probabilities_per_mutation, samples=samples, input_type=input_type, denovo_refit_option=denovo_refit_option)
         
-        with open(output+"JOB_METADATA.txt","a") as sysdata:
-            current_time_end = datetime.datetime.now()
-
-            sysdata.write(f"\n Finished Denovo fitting! \nExecution time:{str(current_time_end-current_time_start)}\n")
+        if make_metadata:
+            with open(os.path.join(output,"JOB_METADATA_SPA.txt"),"a") as sysdata:
+                current_time_end = datetime.datetime.now()
+                sysdata.write(f"\n Finished Denovo fitting! \nExecution time:{str(current_time_end-current_time_start)}\n")
         
 
                                                                 #################
@@ -492,8 +499,9 @@ def spa_analyze(samples, output, input_type='matrix', context_type="96", signatu
                 
         print("\n Decomposing De Novo Signatures  .....")
         current_time_start = datetime.datetime.now()
-        with open(output+"JOB_METADATA.txt","a") as sysdata:
-            sysdata.write("\n Decomposing De Novo Signatures  .....")
+        if make_metadata:
+            with open(os.path.join(output,"JOB_METADATA_SPA.txt"),"a") as sysdata:
+                sysdata.write("\n Decomposing De Novo Signatures  .....")
         final_signatures = sub.signature_decomposition(processAvg, mutation_type, layer_directory2, genome_build=genome_build,cosmic_version=cosmic_version,signature_database=signature_database, mutation_context=mutation_context, add_penalty=0.05, connected_sigs=connected_sigs,remove_penalty=0.01, make_decomposition_plots=make_decomposition_plots, originalProcessAvg=originalProcessAvg,new_signature_thresh_hold=new_signature_thresh_hold,sig_exclusion_list=sig_exclusion_list,exome=exome, m_for_subgroups=m_for_subgroups)    
         #final_signatures = sub.signature_decomposition(processAvg, m, layer_directory2, genome_build=genome_build)
         # extract the global signatures and new signatures from the final_signatures dictionary
@@ -508,8 +516,9 @@ def spa_analyze(samples, output, input_type='matrix', context_type="96", signatu
         colnames = genomes.columns
 
         print("\n Assigning decomposed signature")
-        with open(output+"JOB_METADATA.txt","a") as sysdata:
-            sysdata.write("\n [{}] Assigning decomposed signature \n".format(str(start_time).split(".")[0]))
+        if make_metadata:
+            with open(os.path.join(output,"JOB_METADATA_SPA.txt"),"a") as sysdata:
+                sysdata.write("\n [{}] Assigning decomposed signature \n".format(str(start_time).split(".")[0]))
             
         result = sub.make_final_solution(processAvg, genomes, allsigids, layer_directory2, mutation_type, index, colnames, 
                                 cosmic_sigs=True, attribution = attribution, denovo_exposureAvg  = exposureAvg, sequence=sequence,  
@@ -523,10 +532,10 @@ def spa_analyze(samples, output, input_type='matrix', context_type="96", signatu
                                 export_probabilities_per_mutation=export_probabilities_per_mutation,
                                 samples=samples, input_type=input_type, denovo_refit_option=denovo_refit_option)
 
-        with open(output+"JOB_METADATA.txt","a") as sysdata:
-            current_time_end = datetime.datetime.now()
-
-            sysdata.write(f"\n Finished Decompose fitting! \nExecution time:{str(current_time_end-current_time_start)}\n")
+        if make_metadata:
+            with open(os.path.join(output,"JOB_METADATA_SPA.txt"),"a") as sysdata:
+                current_time_end = datetime.datetime.now()
+                sysdata.write(f"\n Finished Decompose fitting! \nExecution time:{str(current_time_end-current_time_start)}\n")
                                                                 #################
                                                                 # Cosmic Fitting       
                                                                 #################
@@ -621,8 +630,9 @@ def spa_analyze(samples, output, input_type='matrix', context_type="96", signatu
         exposureAvg_dummy = pd.DataFrame(np.random.rand(processAvg.shape[1],genomes.shape[1]),index=allsigids,columns=colnames.to_list()).transpose().rename_axis('Samples')
         print("Assigning COSMIC sigs or Signature Database ...... ")
         current_time_start = datetime.datetime.now()
-        with open(output+"JOB_METADATA.txt","a") as sysdata:
-            sysdata.write("\n Assigning COSMIC sigs or Signature Database ...... ")
+        if make_metadata:
+            with open(os.path.join(output,"JOB_METADATA_SPA.txt"),"a") as sysdata:
+                sysdata.write("\n Assigning COSMIC sigs or Signature Database ...... ")
         if processAvg.shape[0] != 96:
             if genomes.shape[0] == processAvg.shape[0] and collapse_to_SBS96 ==True:
                 sys.exit("Signatures Database and Samples are of same context type and is not equal to 96. please rerun by setting the flag \"collapse_to_SBS96 = False \"")
@@ -639,14 +649,13 @@ def spa_analyze(samples, output, input_type='matrix', context_type="96", signatu
                             export_probabilities=export_probabilities,
                             export_probabilities_per_mutation=export_probabilities_per_mutation,
                             samples=samples, input_type=input_type, denovo_refit_option=denovo_refit_option)
-        
-        with open(output+"JOB_METADATA.txt","a") as sysdata:
-            current_time_end = datetime.datetime.now()
-
-            sysdata.write(f"\n Finished Cosmic fitting! \nExecution time:{str(current_time_end-current_time_start)}\n")
+        if make_metadata:
+            with open(os.path.join(output,"JOB_METADATA_SPA.txt"),"a") as sysdata:
+                current_time_end = datetime.datetime.now()
+                sysdata.write(f"\n Finished Cosmic fitting! \nExecution time:{str(current_time_end-current_time_start)}\n")
     
     # Generate sample reconstruction plots
-    if sample_reconstruction_plots and context_type == "96":
+    if sample_reconstruction_plots and mutation_type == "96":
         ss_recon_odir = os.path.join(
                     layer_directory3,
                     "Activities",
@@ -665,15 +674,15 @@ def spa_analyze(samples, output, input_type='matrix', context_type="96", signatu
                                         ss_recon_odir,
                                         execution_parameters)
 
-    # Complete JOB_METADATA
-    sysdata = open(output+"/JOB_METADATA.txt", "a")
-    end_time = datetime.datetime.now()
-    sysdata.write("\n[{}] Analysis ended: \n".format(str(end_time).split(".")[0]))
-    sysdata.write("\n-------Job Status------- \n")
-    sysdata.write("Assignment of mutational signatures completed successfully! \nTotal execution time: "+str(end_time-start_time).split(".")[0]+" \nResults can be found in: "+" "+output+ " " +" folder")
-    sysdata.close()
-
-    print("\n\n \nYour Job Is Successfully Completed! Thank You For Using SigProfilerAssignment.\n ")
+    # Complete JOB_METADATA_SPA
+    if make_metadata:
+        sysdata = open(os.path.join(output,"JOB_METADATA_SPA.txt"), "a")
+        end_time = datetime.datetime.now()
+        sysdata.write("\n[{}] Analysis ended: \n".format(str(end_time).split(".")[0]))
+        sysdata.write("\n-------Job Status------- \n")
+        sysdata.write("Assignment of mutational signatures completed successfully! \nTotal execution time: "+str(end_time-start_time).split(".")[0]+" \nResults can be found in: "+" "+output+ " " +" folder")
+        sysdata.close()
+        print("\n\n \nYour Job Is Successfully Completed! Thank You For Using SigProfilerAssignment.\n ")
   
 
 
